#!/bin/bash
target="$PWD"
export PATH="$target/usr/bin:$PATH"
export CPPFLAGS="-I$target/usr/include"
export CFLAGS="-O2 -arch i386 -m32 -I$target/usr/include"
export CXXFLAGS="$CFLAGS "
export LDFLAGS=" -arch i386 -L$target/usr/lib"
export PKG_CONFIG_PATH="$target/usr/lib/pkgconfig"

## Download (unless file already exists), extract, configure and build
##  $1  target folder to run configure (as defined in the download file)
##  $2  download location
##  $3+ optional configure parameters (--prefix automatically set) 
download_build() {
 file=$(basename "$2") 
 [[ ! -f $file ]] && curl -L $2 -o "$file"
 [[ ! -d $1 ]] && tar xf "$file"
 pushd "$1" > /dev/null
 ./configure --prefix="$target/usr" ${@:3}
 make uninstall
 make -j5 install
 make install
 popd > /dev/null
}
## Relink executable files, with very specific "bin/" filter for samba 3 (samba 4 is worse)
##  $1  folder containing executable files
##  $2  optional file name filter
relink_bin() {
 pushd "$1" > /dev/null
 for file in "$2"*; do
  if [[ ! -L "$file" && "$file" != *".old" && $(file "$file") == *"Mach-O"* ]]; then
   echo Relink $file
   while read -r full; do 
    dir=$(dirname "$full")
    name=$(basename "$full") 
    [[ "$full" == "$target/usr/lib/$name" ]] && install_name_tool -change "$full" "@loader_path/../lib/$name" "$file"
    [[ "$full" == "bin/$name" ]] && install_name_tool -change "$full" "@loader_path/../lib/$name" "$file"
   done < <(otool -L "$file" | tail -n +2 | cut -d'(' -f1)
   fi
 done
 popd > /dev/null
}
## Relink dynamic library files, slightly different to replace id's
##  $1  folder containing library files
##  $2  optional file name filter
relink_lib() {
 pushd "$1" > /dev/null
 for file in "$2"*; do
  if [[ ! -L "$file" && "$file" != *".old" && $(file "$file") == *"Mach-O"* ]]; then
   while read -r full; do 
    dir=$(dirname "$full")
    name=$(basename "$full") 
    [[ "$full" == *"/$file" ]] && install_name_tool -id "$name" "$file"
    [[ "$full" != *"/$file" && "$full" == "$target/usr/lib/$name" ]] && install_name_tool -change "$full" "@loader_path/../lib/$name" "$file"
    [[ "$full" == "bin/$name" ]] && install_name_tool -change "$full" "@loader_path/../lib/$name" "$file"
   done < <(otool -L "$file" | tail -n +2 | cut -d'(' -f1)
   fi
 done
 popd > /dev/null
}
## List links within an executable or dynamic library
##  $1  target folder
##  $2  optional file name filter
list_links() {
 pushd "$1" > /dev/null
 for file in "$2"*; do
  if [[ ! -L "$file" && "$file" != *".old" && $(file "$file") == *"Mach-O"* ]]; then
   otool -L "$file"
  fi
 done
 popd > /dev/null
}
## Update last section of default wine.inf (registry), typically LicenseManager for retina mode
##  Lazy, assumptions. 3rd param typically 0 (string) or 0x00010001 (dword)
##  $1  location where share/wine/wine.inf resides
retina_wineinf() {
 echo 'HKCU,Software\Wine\Mac Driver,"RetinaMode",,"y"
HKCU,Control Panel\Desktop,"FontSmoothing",,"2"
HKCU,Control Panel\Desktop,"FontSmoothingOrientation",0x00010001,0x00000001
HKCU,Control Panel\Desktop,"FontSmoothingType",0x00010001,0x00000002
HKCU,Control Panel\Desktop,"FontSmoothingGamma",0x00010001,0x00000578
HKLM,System\CurrentControlSet\Hardware Profiles\Current\Software\Fonts,"LogPixels",0x00010001,0x000000cc
' >> "$1/share/wine/wine.inf"
}
## Create wine.app with 2 mandatory files, 2 custom files and other Resources
##  $1  name and location of wine.app
##  $2  location of 2 custom files "startwine" and "wine.icns"
create_app() {
 mkdir -p $1/Contents/MacOS
 mkdir -p $1/Contents/Resources
 pushd "$1/Contents" > /dev/null
 echo "APPL????" > PkgInfo
 echo '<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0"><dict>
<key>CFBundleExecutable</key><string>startwine</string>
<key>CFBundleIconFile</key><string>wine.icns</string>
<key>CFBundleIdentifier</key><string>com.myByways.wine</string>
<key>CFBundlePackageType</key><string>APPL</string>
<key>CFBundleVersion</key><string>1.9.21</string>
<key>CFBundleSupportedPlatforms</key><array><string>MacOSX</string></array>
<key>LSApplicationCategoryType</key><string>public.app-category.utilities</string>
<key>LSMinimumSystemVersion</key><string>10.6</string>
</dict></plist>' > Info.plist
 cd Resources
 cp -R "$target/usr/bin" .
 cp -R "$target/usr/lib" .
 mkdir share
 cp -R "$target/usr/share/wine" share
 rmdir lib/gpext lib/nss_info lib/pdb lib/perfcount 
 cp "$target/custom/startwine" ../MacOS
 cp "$target/custom/wine.icns" .
 popd > /dev/null
}
## Create wine.app with 2 mandatory files, 2 custom files and other Resources
##  $1  name and location of wine.app
##  $2  location of 2 custom files "startwine" and "wine.icns"
create_app_relink() {
 create_app "$1" "$2"
 relink_bin "$1/Contents/Resources/bin"
 relink_lib "$1/Contents/Resources/lib"
}



## Mandatory libraries
## Sources: http://libpng.org http://freetype.org http://ijg.org
download_build libpng-1.6.29 http://downloads.sourceforge.net/project/libpng/libpng16/1.6.29/libpng-1.6.29.tar.gz --disable-static
download_build freetype-2.7.1 http://download.savannah.gnu.org/releases/freetype/freetype-2.7.1.tar.gz --disable-static
download_build jpeg-9b http://ijg.org/files/jpegsrc.v9b.tar.gz --disable-static

## Optional libraries
## Sources: http://libtiff.org http://www.littlecms.com http://www.unixodbc.org/download.html
download_build tiff-3.8.2 http://dl.maptools.org/dl/libtiff/tiff-3.8.2.tar.gz --disable-static
download_build lcms2-2.8 http://freefr.dl.sourceforge.net/project/lcms/lcms/2.8/lcms2-2.8.tar.gz --disable-static
download_build unixODBC-2.3.4 http://www.unixodbc.org/unixODBC-2.3.4.tar.gz --disable-static

## Mandatory samba (ntlm is required for Word authentication)
download_build samba-3.6.25/source3 https://ftp.samba.org/pub/samba/samba-3.6.25.tar.gz  --sbindir="$target/usr/bin" --disable-largefile --disable-swat --disable-smbtorture4 --disable-cups --disable-pie --disable-relro --disable-external-libtalloc --disable-external-libtdb --disable-fam --disable-dnssd --disable-avahi --without-dmapi --without-ldap --without-dnsupdate --without-pam --without-pam_smbpass --without-utmp --without-cluster-support --without-acl-support --without-sendfile-support --with-included-popt --with-included-iniparser --with-winbind 

## Build Wine
download_build wine-2.9 http://dl.winehq.org/wine/source/2.x/wine-2.9.tar.xz --enable-win16 --disable-win64 --without-x

## Create app and relink libraries
app="$target/wine29.app"
create_app_relink "$app" "$target/custom"

## Tweak to enable retina mode properly - comment out if not applicable!
retina_wineinf "$app/Contents/Resources"
